import numpy as np


def mergenumbers(values, quiet=False):
    if type(values) == list:
        values = np.array(values)

    weight = sum(1 / values[:,1]**2)
    value = sum(values[:,0] / values[:,1]**2 / weight)
    chi = 1./len(values) * sum(
        ((values[:,0]-value) / values[:,1])**2
    )
    ans = np.array([value, np.sqrt(1/weight)])

    if not quiet:
        print chi
        return ans
    else:
        return chi, ans


def plusnumbers(*args):
    if len(args) == 1:
        lst = np.array(args[0])
    else:
        lst = np.array(args)
    return np.array([
        sum(lst[:,0]),
        np.sqrt(sum(lst[:,1]**2))
    ])


def dividenumbers(a, b):
    return np.array([
        a[0]/b[0],
        np.sqrt((b[1]**2*a[0]**2)/b[0]**4 + a[1]**2/b[0]**2)
    ])


def timesnumbers(a, b):
    return np.array([
        a[0]*b[0],
        np.sqrt(b[0]**2*a[1]**2 + a[0]**2*b[1]**2)
    ])


def integratehistogram(hist):
    ans = 0
    if hist[0,0] == -np.inf:
        ans += hist[0,1]
        hist = hist[1:]
    if hist[-1,0] == +np.inf:
        ans += hist[-1,1]
        hist = hist[:-1]

    ans += np.mean(np.diff(hist[:,0])) * sum(hist[:,1])
    return ans


def mergeplots(ps, returnchi=False):
    if type(ps) == list:
        ps = np.array(ps)

    with np.errstate(divide='ignore', invalid='ignore'):
        w = sum(1/ps[:,:,2]**2)
        out = np.zeros(ps[0].shape)
        out[:,0] = ps[0,:,0]

        out[:,1] = sum(ps[:,:,1]/ps[:,:,2]**2 / w)
        out[:,2] = np.sqrt(1/w)

        # if one bin is empty, the zero causes trouble
        out[~np.isfinite(out)] = 0

        chi = 1./(len(ps)-1)*sum((ps[:,:,1]-out[:,1])**2/ps[:,:,2]**2)
        chi = np.column_stack((out[:,0], chi))
        chi[~np.isfinite(chi)] = 0

    if returnchi:
        return out, chi
    else:
        return out


def combineplots(a, b, yfunc, efunc):
    # There must be a better way of doing this
    maskA = [False]*len(a)
    maskB = [False]*len(b)
    for ia in range(len(a)):
        for ib in range(len(b)):
            if abs(1-a[ia, 0] / b[ib, 0]) < 1e-10:
                maskA[ia] = True
                maskB[ib] = True

    with np.errstate(divide='ignore', invalid='ignore'):
        x = a[maskA, 0]
        y = yfunc(a[maskA, 1], b[maskB, 1])
        e = efunc(a[maskA, 1], a[maskA, 2], b[maskB, 1], b[maskB, 2])

        out = np.column_stack((x, y, e))
        # if one bin is empty, the zero causes trouble
        out[~np.isfinite(out)] = 0

    return out


def addplots(a, b, sa=1., sb=1.):
    return combineplots(
        a, b,
        lambda y1, y2: y1 + y2,
        lambda y1, e1, y2, e2: np.sqrt(e1**2 + e2**2)
    )


def divideplots(a, b):
    return combineplots(
        a, b,
        lambda y1, y2: y1 / y2,
        lambda y1, e1, y2, e2: np.sqrt(e2**2 * y1**2 / y2**4 + e1**2 / y2**2)
    )


def combineNplots(func, plots):
    accum = plots[0]
    for plot in plots[1:]:
        accum = func(accum, plot)
    return accum


def scaleplot(a, sx, sy=None):
    if sy is None:
        return np.array((1./sx, sx, sx)) * a
    else:
        return np.array((1./sx, sy, sy)) * a


def mergebins(p, n):
    if p[0,0] == -np.inf:
        return np.concatenate([
            [p[0,:]],
            mergebins(p[1:,:], n)
        ])
    if p[-1,0] == +np.inf:
        return np.concatenate([
            mergebins(p[:-1,:], n),
            [p[-1,:]]
        ])

    # Make sure the length fits
    if len(p) % n:
        short = p[:-(len(p) % n)]
    else:
        short = p[:]
    part = np.split(short, len(short)/n)
    return np.array([
        (
            sum(i[:,0])/n,
            sum(i[:,1])/n,
            np.sqrt(sum(i[:,2]**2))/n
        ) for i in part
    ])
